/*
 * DataDada - is an application that will turn the stored data on 
 * your hard drive into a movie complete with sound, image, and 
 * subtitles.  
 * 
 * Copyright (C) 2003 August Black
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * DataAudio.cpp
 */

#include <math.h>   // for sin function to fill audio table, only for testing
#include <stdio.h>  // for fprintf
#include <stdlib.h> // for int rand()
#include "DataAudio.h"



DataAudio :: DataAudio()
{ 
  sample_rate = 44100;
  samples_per_frame = 2;  // stereo
  frames_per_block = 64;  // we could be more conservative here.
  sample_size = sizeof(short);
  fifo = new Fifo(frames_per_block*1000*sample_size);  
  quit = false;
  mut = SDL_CreateMutex();
}

DataAudio :: ~DataAudio()
{
}



int pa_callback(  void *inputBuffer, void *outputBuffer,
                     unsigned long framesPerBuffer,
                     PaTimestamp outTime, void *userData )
{
  DataAudio *data = (DataAudio *)userData;
  unsigned int len, tmp;
  len = tmp = data->getSamplesPerFrame() * framesPerBuffer  * data->getSampleSize() ;
  short * out;

  (void) outTime; /* Prevent unused variable warnings. */
  (void) inputBuffer;

  data->FifoRead((void *)outputBuffer, len);
  if ( len < tmp ) {
    //fprintf( stderr, "len = %d, outbuf = %d \n", len, tmp );
    out = (short *)outputBuffer;
    for(unsigned int i=0; i< framesPerBuffer; i++ )
      {
	*out++ =  0; // left 
	*out++ =  0; // right 
      } 
  } 
  
  return 0;
}


void DataAudio :: Run()  
{
  quit = false;
  
  //fprintf( stderr, "Starting Audio\n" );
  
  err = Pa_Initialize();
  //fprintf( stderr, "Initialized Audio\n" );
  err = Pa_OpenDefaultStream(
			     &pa_stream,           /* passes back stream pointer */
			     0,                 /* no input channels */
			     samples_per_frame, /* stereo output */
			     paInt16,         /* paFloat32, 32 bit floating point output */
			     sample_rate,       /* sample rate */
			     frames_per_block,  /* frames per buffer */
			     0,                 /* number of buffers, if zero then use default minimum */
			     pa_callback,       /* specify our custom callback */
			     this); 
  //fprintf( stderr, "Opened Default Audio\n" );
  err = Pa_StartStream( pa_stream );
  
  if( err != paNoError ) {
    Error();
  }
  
  //fprintf( stderr, "Checked for Errors\n" );
}

void DataAudio::Quit() {
  PaError err;
  err = Pa_StopStream( pa_stream );
  if( err != paNoError ) Error();
  err = Pa_CloseStream( pa_stream );
  err = Pa_StopStream( pa_stream );
  Pa_Terminate();
}

void DataAudio::Error() {
  
  fprintf( stderr, "An error occured while using the portaudio stream\n" );
  fprintf( stderr, "Error number: %d\n", err );
  fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
  Pa_Terminate();
  //return err;
}

int DataAudio::FifoRead(void *buf, unsigned int &len) {
  
  SDL_mutexP(mut);    
  fifo->Read(buf, len); 
  SDL_mutexV(mut);
  return 0;
}

unsigned int DataAudio::FreeSpace() {
  unsigned int free;
  SDL_mutexP(mut);    
  free = fifo->FreeSpace();
  SDL_mutexV(mut);
  return free;
}

int DataAudio::FifoWrite(void *buf, unsigned int len) {
  int ret;
  SDL_mutexP(mut);    /* Lock  the potty */
  ret = fifo->Write(buf, len); 
  SDL_mutexV(mut);
  return ret;
}
