/*
 * DataDada - is an application that will turn the stored data on 
 * your hard drive into a movie complete with sound, image, and 
 * subtitles.  
 * 
 * Copyright (C) 2003 August Black
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * DataDada.cpp
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>   // for sin function to fill audio table, only for testing

#include <sys/types.h>  //open, lseek, etc.
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef WIN
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <string.h>
#include "SDL.h"
#endif

#include "DataVideo.h"
#include "DataAudio.h"
#include "Flist.h"

extern "C" {
    #include "SFont.h"
}
#include <string>
#include <iostream>
using namespace std;


#if SDL_BYTEORDER == SDL_BIG_ENDIAN
#define rmask 0xff000000
#define gmask 0x00ff0000
#define bmask 0x0000ff00
#define amask 0x000000ff
#else 
#define rmask 0x000000ff //rmask = 0x000000ff;
#define gmask 0x0000ff00 //gmask = 0x0000ff0;
#define bmask 0x00ff0000 //bmask = 0x00ff0000;
#define amask 0xff000000
#endif


//#define SCALE (1./(1024. * 1024. * 1024. * 2.))
int UpdateVideo( DataVideo *dv,   SDL_Surface *blittmp );
int UpdateFilename( DataVideo *dv,   SDL_Surface *blittmp );
void DoEvents( DataVideo *dv,   SDL_Surface *blittmp );


int main(int argc, char *argv[]){
  
  SDL_Surface *blittmp = NULL;
  DataVideo *datavid = new DataVideo();
  DataAudio *datasnd = new DataAudio();
  Flist *fl = new Flist();
  string filename;
  int fd;
  int ret;
  int len = 128;
  short buf[len];

  if (argc > 1) {
    fl->Run(argv[1]);
  } else {
#ifdef WIN
    fl->Run("G:");
#else
    fl->Run("/");
#endif
  }


  SDL_Delay(500); //wait half a  second or so to let Flist get a listing
  
  //fl->PrintList();
  fd = -1;
  while ( fd == -1) {
    filename = fl->getNextItem();
    //return and exit if we are at end of list
    if ( filename == fl->getLastItem()) { cout<< "last item reached" << endl; return 1; }  
    fd = open( filename.c_str(),  O_RDONLY ); 
    if (fd == -1) {
      fprintf(stderr, "couldnt open file %s, trying again\n", filename.c_str());
      close(fd);
    }
  }	


  datavid->Init(480,240,32);
  // start out with black
  blittmp = SDL_CreateRGBSurfaceFrom( datavid->getPixels(), datavid->getWidth(), datavid->getHeight(), 
				      datavid->getBpp(), ( datavid->getWidth()* 4), 
				      rmask, gmask, bmask, 0); 
    /*blittmp = SDL_CreateRGBSurfaceFrom( datavid->getPixels(), 800, 600, 
				      datavid->getBpp(), ( 800* 4), 
				      rmask, gmask, bmask, 0); */
  SDL_FillRect(blittmp,NULL, 0);
  
  // start the audio thread
  datasnd->Run();
  
  while ( !datavid->shouldQuit() ) {
    
    while ( datasnd->FreeSpace( ) >= (unsigned int)len ) {
      ret = read(fd, buf, len );
      if (ret < len) {  // reached end of file
	close(fd); fd = -1;
	while ( fd == -1) {
	  filename = fl->getNextItem();
	  if ( filename == fl->getLastItem()) {
	    //cerr << "last f#$@ing item" <<endl;
	    datavid->Quit();
	    datasnd->Quit();
	    SDL_Quit();
	    return 1;  //return and exit if we are at end of list
	  }
	  fd = open( filename.c_str(),  O_RDONLY ); 
	 
#ifdef DEBUG
	  if (fd == -1) 
	    cout << "       couldn't open " << filename.c_str() << endl;
#endif
	}	
	//UpdateFilename( datavid, blittmp );
      }
      
      datasnd->FifoWrite( (void *)buf, ret);
      datavid->FifoWrite( (void *)buf, ret);
      datavid->setFilename( filename.c_str() );
      DoEvents( datavid, blittmp );
    }
    
    UpdateVideo( datavid, blittmp );
    //SDL_CondWait(audiocond, mut);
    SDL_Delay(60);
  } 
  
  //printf("quitting in main, why?\n");
  datavid->Quit();
  datasnd->Quit();
  SDL_Quit();
  return 1;
}

int UpdateVideo( DataVideo *dv , SDL_Surface *blittmp) 
{

  if ( dv->UsedSpace() >= (unsigned int)dv->getMaxsize() ) {
    // read data into pixels
    //cerr <<  "used space = " << dv->UsedSpace() << " maxsize = " << dv->getMaxsize() 
    // << " freespace = " << dv->FreeSpace() << endl;
    dv->FifoRead();
    //if (blittmp != NULL) 
    //SDL_FreeSurface(blittmp);
    blittmp = SDL_CreateRGBSurfaceFrom( dv->getPixels(), dv->getWidth(), dv->getHeight(), 
					dv->getBpp(), ( (dv->getWidth()) * 4),
					rmask, gmask, bmask, 0);
    
    //blit to screen
    if (blittmp != NULL) {
      if( SDL_BlitSurface(blittmp, NULL, dv->getScreen(), NULL) < 0)
	cerr << "BlitSurface error: " << SDL_GetError() << endl;
    }
    
    //always blit the string
    XCenteredString( dv->getScreen(), dv->getScreen()->h - 20 , dv->getFilename());
    SDL_Flip( dv->getScreen() );   
  } else {
    UpdateFilename( dv, blittmp );
  }
  
  return 1;
}


int UpdateFilename( DataVideo *dv , SDL_Surface *blittmp) 
{
  if (blittmp != NULL) {
    if( SDL_BlitSurface(blittmp, NULL, dv->getScreen(), NULL) < 0)
      cerr << "BlitSurface error: " << SDL_GetError() << endl;
  } else {
    cerr << "BlitSurface is NULL " << endl;
  }
  XCenteredString( dv->getScreen(), dv->getScreen()->h - 20 , dv->getFilename());
  SDL_UpdateRect( dv->getScreen(), 0, dv->getScreen()->h-20, dv->getScreen()->w, 20);
  return 1;
}

void DoEvents(  DataVideo *dv, SDL_Surface *blittmp ) {
  SDL_Event E;
  SDL_PollEvent(&E);
  switch (E.type) {
    case SDL_KEYDOWN:
	if (E.key.keysym.sym == SDLK_q ) {
          dv->Quit();
	}
	if (E.key.keysym.sym == SDLK_f ) {
	  dv->ToggleFullscreen();
	}
	if (E.key.keysym.sym == SDLK_LEFT ) {
	  dv->ReInit(dv->getWidth()-16, dv->getHeight() );
	}
	if (E.key.keysym.sym == SDLK_RIGHT ) {
	  dv->ReInit(dv->getWidth()+16, dv->getHeight() );
	}
	if (E.key.keysym.sym == SDLK_UP ) {
	  dv->ReInit(dv->getWidth(), dv->getHeight()-16 );
	}
	if (E.key.keysym.sym == SDLK_DOWN ) {
	  dv->ReInit(dv->getWidth(), dv->getHeight()+16 );
	}
        
        if (blittmp != NULL) {
	  SDL_FreeSurface(blittmp);
	  blittmp = NULL;
	  blittmp = SDL_CreateRGBSurfaceFrom( dv->getPixels(), dv->getWidth(), dv->getHeight(), 
					      dv->getBpp(), ( (dv->getWidth()) * 4),
					      rmask, gmask, bmask, 0);
	  if( SDL_BlitSurface(blittmp, NULL, dv->getScreen(), NULL) < 0)
	    cerr << "BlitSurface error: " << SDL_GetError() << endl;
	}
	XCenteredString( dv->getScreen(), dv->getScreen()->h - 20 , dv->getFilename());
	SDL_Flip( dv->getScreen() ); 
  }

}
