/*
 * DataDada - is an application that will turn the stored data on 
 * your hard drive into a movie complete with sound, image, and 
 * subtitles.  
 * 
 * Copyright (C) 2003 August Black
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * DataVideo.cpp
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <string>
#include <iostream>

#include "DataVideo.h"
#include "font.h"

using namespace std;

DataVideo :: DataVideo()
{ 

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
  unsigned int rmask = 0xff000000;
  unsigned int gmask = 0x00ff0000;
  unsigned int bmask = 0x0000ff00;
  unsigned int amask = 0x000000ff;
#else 
   unsigned int rmask = 0x000000ff;
   unsigned int gmask = 0x0000ff00;
   unsigned int bmask = 0x00ff0000;
   unsigned int amask = 0xff000000;
#endif

  quit = false;
  pixels = NULL;
  fifo = NULL;
  width = 800;
  height = 600;
  bpp = 32;
  maxsize = 800*600*32/8;
  fifo = new Fifo(MAXSIZE+(MAXSIZE/2));    
  if ( (pixels = (char *)malloc( MAXSIZE )) == NULL)
    cerr << "Couldn't malloc enough space" << endl;
#ifndef WIN
  bzero( (void *)pixels , sizeof(pixels) );
#endif
  font = SDL_CreateRGBSurfaceFrom((void *)FONT_PIXEL_DATA, FONT_WIDTH, FONT_HEIGHT, 32, 
				  FONT_WIDTH*FONT_BYTES_PER_PIXEL, rmask, gmask, bmask, amask);
  InitFont(font);
  
  //sdlflags = SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF|SDL_ASYNCBLIT|SDL_ANYFORMAT ;
  //sdlflags = SDL_SWSURFACE|SDL_ANYFORMAT ;
  sdlflags = SDL_SWSURFACE ;
  full = false;
}

void DataVideo :: ToggleFullscreen() {
  if ( full ) {
    full = false;
    sdlflags = SDL_SWSURFACE|SDL_ANYFORMAT ;
  } else {
    full = true;
    sdlflags = SDL_SWSURFACE|SDL_FULLSCREEN|SDL_ANYFORMAT ;
  }
  this->ReInit(width, height );
}

DataVideo :: ~DataVideo() 
{
}


void DataVideo :: Quit() {
  quit = true; 
}

bool DataVideo :: shouldQuit() {
  bool tmp;
  tmp = quit;
  return tmp;
}


void DataVideo::ReInit( int W, int H) {
  width = W;
  height = H;
  if (width > MAXWIDTH) width = MAXWIDTH;
  if (width < MINWIDTH) width = MINWIDTH;
  if (height > MAXHEIGHT) height = MAXHEIGHT;
  if (height < MINHEIGHT) height = MINHEIGHT;

  setMaxsize(); //maxsize = width*height*bpp/8;
  screen = SDL_SetVideoMode( width, height, bpp, sdlflags);
  
  if ( screen == NULL ) {
    cerr << "Unable to set video size/resolution " << SDL_GetError() << endl;
  }

}

void DataVideo::Init( int W, int H, int B) {
  
  width = W;
  height = H;
  bpp = B;
  if (width > MAXWIDTH) width = MAXWIDTH;
  if (width < MINWIDTH) width = MINWIDTH;
  if (height > MAXHEIGHT) height = MAXHEIGHT;
  if (height < MINHEIGHT) height = MINHEIGHT;
  if (bpp > MAXBPP) bpp = MAXBPP;
  if (bpp < MINBPP) bpp = MINBPP;

  maxsize = width*height*bpp/8;

  if ( SDL_Init(SDL_INIT_VIDEO) < 0) {
    cerr << "Unable to init SDL: " << SDL_GetError() << endl;
  } 
  
  atexit(SDL_Quit);


  screen = SDL_SetVideoMode( width, height, bpp, sdlflags);

  if (SDL_VideoModeOK( width, height, bpp, SDL_SWSURFACE) == 0) {
    cerr << "Video mode not ok" << endl;
  } else {
    //BPP = SDL_VideoModeOK(WIDTH, HEIGHT, BPP, SDL_SWSURFACE);
#ifdef DEBUG
    cerr << "Video mode OK, bpp = " <<  SDL_VideoModeOK( width , height, bpp, SDL_SWSURFACE)  << endl;
#endif
  }
  
  screen = SDL_SetVideoMode( width, height, bpp, sdlflags);
  if ( screen == NULL ) {
    cerr << "Unable to set video size/resolution " << SDL_GetError() << endl;
  }
  SDL_WM_SetCaption( "DataDada", "DataDada");
#ifdef DEBUG
  cout << "Video details successfully initialized" << bpp << " bpp " << endl;
#endif
}

int DataVideo::FifoRead() {
  unsigned int len,tmp;
  len = tmp = maxsize;
  fifo->Read( (void *)pixels, len);  
#ifdef DEBUG
  if ( len != tmp ) cerr << "Problem reading from fifo, len = " << len << " tmp = " << tmp 
			 << " maxsize = " << maxsize <<endl;
#endif
  return len;
}

int DataVideo::FifoRead(void *buf, unsigned int &len) {
  fifo->Read(buf, len);  
  return len;
}


int DataVideo::FifoWrite(void *buf, unsigned int len) {
  int ret;
  ret = fifo->Write(buf, len); 
  return ret;
}

unsigned int DataVideo::FreeSpace() {
  unsigned int free;   
  free = fifo->FreeSpace();
  return free;
}

unsigned int DataVideo::UsedSpace() {
  unsigned int used;   
  used = fifo->UsedSpace();
  return used;
}


void DataVideo::setFilename(const char *fn) {
  strcpy(filename, fn);
}

char * DataVideo::getFilename() {
  char * fn;
  fn = filename;
  return fn;
}
