/*
 * DataDada - is an application that will turn the stored data on 
 * your hard drive into a movie complete with sound, image, and 
 * subtitles.  
 * 
 * Copyright (C) 2003 August Black
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * DataVideo.h
 */
#ifndef _DATAVIDEO_H_
#define _DATAVIDEO_H_

#include <memory.h>
#include "Fifo.h"

extern "C" {
#include <SDL.h>
#include <SDL_thread.h>
#include "SDL_mutex.h"
#include "SFont.h"
}

#define MAXWIDTH 800
#define MINWIDTH 100
#define MAXHEIGHT 600
#define MINHEIGHT 100
#define MAXBPP 32
#define MINBPP 16
#define MAXSIZE MAXWIDTH*MAXHEIGHT*MAXBPP/8

class DataVideo
{
 public:
  DataVideo();
  //DataVideo(int width, int height, int bpp);
  ~DataVideo();

  void Init(int w, int h, int bpp);
  void ReInit(int w, int h);

  void Quit();
  bool shouldQuit();
  unsigned int UsedSpace();
  unsigned int FreeSpace();
  int FifoRead();
  int FifoRead(void *buf, unsigned int &len);
  int FifoWrite( void *buf, unsigned int len );

  char * getFilename();
  void setFilename(const char *fn);

  int getBpp() { return bpp; }
  int getWidth() { return width; }
  int getHeight() { return height; }
  int getMaxsize() { return maxsize; }
  char *getPixels() { return pixels; }
  SDL_Surface *getScreen() { return screen; }
  SDL_Surface *getFont() { return font; }

  
  void ToggleFullscreen();
  void setBpp( int x ) { bpp = x; }
  void setWidth( int x ) { width = x; }
  void setHeight( int x ) { height = x; }
  void setMaxsize( int x ) { maxsize = x; if (maxsize > MAXSIZE) maxsize = MAXSIZE; }
  void setMaxsize(  ) { maxsize = width * height * bpp / 8; if (maxsize > MAXSIZE) maxsize = MAXSIZE;}

 private:
  int bpp;
  int width;
  int height;
  int maxsize;
  char *pixels;
  char filename[1024];
  bool quit;
  bool full;
  Uint32 sdlflags;
  Fifo *fifo;
  SDL_Surface *screen;
  SDL_Surface *font;

};
#endif

